#include "ActiveObject.hpp"


//constructor
Mutex::Mutex() {
    pthread_mutex_init(&m_mutex, 0);
}


//destructor
Mutex::~Mutex() {
    pthread_mutex_destroy(&m_mutex);
}


//lock
void Mutex::lock() {
    pthread_mutex_lock(&m_mutex);
}


//unlock
void Mutex::unlock() {
    pthread_mutex_unlock(&m_mutex);
}


//constructor
WaitCondition::WaitCondition() {
    pthread_cond_init(&m_cond, 0);
}


//destructor
WaitCondition::~WaitCondition() {
    pthread_cond_destroy(&m_cond);
}


//signal the condition
void WaitCondition::signal() {
    pthread_cond_signal(&m_cond);
}


//wait
void WaitCondition::wait(Mutex &mutex) {
    pthread_cond_wait(&m_cond, &mutex.m_mutex);
}


//constructor
Semaphore::Semaphore() {
    sem_init(&m_sem, 0, 0);
}


//destructor
Semaphore::~Semaphore() {
    sem_destroy(&m_sem);
}


//increment
void Semaphore::operator ++() {
    sem_post(&m_sem);
}


//decrement
void Semaphore::operator --() {
    sem_wait(&m_sem);
}


//constructor
ActiveObject::ActiveObject() {
    pthread_create(&m_thread, 0, &_threadProc, this);
}


//destructor
ActiveObject::~ActiveObject() {
    put(std::bind(&ActiveObject::_stop, this));
    pthread_join(m_thread, 0);
}


//put message in queue
void ActiveObject::put(const std::function<void()> &f) {
    m_queue.put(f);
}


//run
void ActiveObject::_run() {
    m_loop = true;
    while (m_loop) {
        std::function<void()> f;
        m_queue.get(f);
        f();
    }
}


//stop
void ActiveObject::_stop() {
    m_loop = false;
}


void *ActiveObject::_threadProc(void *p) {
    ((ActiveObject *)p)->_run();
    return 0;
}
